#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `psi4_step` package."""

import pytest
import psi4_step
import re  # noqa: F401


@pytest.fixture
def instance():
    instance = psi4_step.Psi4()
    instance._id = (1,)
    return instance


def test_construction():
    """Simplest test that we can make a Psi4 object"""
    instance = psi4_step.Psi4()
    assert str(type(instance)) == "<class 'psi4_step.psi4.Psi4'>"


def test_version():
    """Test that the object returns a version"""
    instance = psi4_step.Psi4()
    result = instance.version
    assert isinstance(result, str) and len(result) > 0


def test_git_revision():
    """Test that the object returns a git revision"""
    instance = psi4_step.Psi4()
    result = instance.git_revision
    assert isinstance(result, str) and len(result) > 0


# def test_description_text_default(instance):
#     """Test the default description text"""

#     print(instance.description_text())
#     assert re.fullmatch(
#         (
#             r'Step 1: Psi4  [-+.0-9a-z]+\n'
#             r'    Create a 2 x 2 x 2 supercell from the current cell'
#         ), instance.description_text()
#     ) is not None



# -*- coding: utf-8 -*-

"""The graphical part of a Psi4 Initialization node"""

import logging
import tkinter as tk

import seamm
import seamm_widgets as sw

logger = logging.getLogger(__name__)


class TkInitialization(seamm.TkNode):
    def __init__(
        self,
        tk_flowchart=None,
        node=None,
        canvas=None,
        x=120,
        y=20,
        w=200,
        h=50,
        my_logger=logger,
    ):
        """Initialize the graphical Tk Psi4 initialization step

        Keyword arguments:
        """
        self.results_widgets = []

        # Set the logging level for this module if requested
        # if 'psi4_tk_initialization_log_level' in self.options:
        #     my_logger.setLevel(self.options.psi4_tk_initialization_log_level)
        #     my_logger.critical(
        #         'Set log level to {}'.format(
        #             self.options.psi4_tk_initialization_log_level
        #         )
        #     )

        super().__init__(
            tk_flowchart=tk_flowchart,
            node=node,
            canvas=canvas,
            x=x,
            y=y,
            w=w,
            h=h,
            my_logger=my_logger,
        )

    def right_click(self, event):
        """Probably need to add our dialog..."""

        super().right_click(event)
        self.popup_menu.add_command(label="Edit..", command=self.edit)

        self.popup_menu.tk_popup(event.x_root, event.y_root, 0)

    def create_dialog(self, title="Edit Psi4 Initialization Step"):
        """Create the dialog!"""
        self.logger.debug("Creating the dialog")
        frame = super().create_dialog(title=title)

        # Create all the widgets
        P = self.node.parameters
        for key in P:
            if key not in ("results", "extra keywords", "create tables"):
                self[key] = P[key].widget(frame)

        self.logger.debug("Finished creating the dialog")

    def reset_dialog(self, widget=None):
        frame = self["frame"]
        for slave in frame.grid_slaves():
            slave.grid_forget()

        widgets = []
        row = 0
        self["basis"].grid(row=row, column=0, sticky=tk.EW)
        widgets.append(self["basis"])
        row += 1
        self["symmetry_tolerance"].grid(row=row, column=0, sticky=tk.EW)
        widgets.append(self["symmetry_tolerance"])
        row += 1
        sw.align_labels(widgets)

        return row



"""This file contains metadata describing the results from Psi4
"""

metadata = {}

methods = {
    "effective fragment potential (EFP)": {
        "method": "efp",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "analytic",
    },
    "Kohn-Sham (KS) density functional theory (DFT)": {
        "method": "dft",
        "calculation": ["energy", "gradients"],
        "level": "normal",
        "gradients": "analytic",
    },
    "Hartree-Fock (HF) self consistent field (SCF)": {
        "method": "hf",
        "calculation": ["energy", "gradients"],
        "level": "normal",
        "gradients": "analytic",
    },
    "HF with dispersion, BSSE, and basis set corrections": {
        "method": "hf3c",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
    },
    "PBEh with dispersion, BSSE, and basis set corrections": {
        "method": "pbeh3c",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
    },
    "density cumulant functional theory": {
        "method": "dcft",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "analytic",
    },
    "2nd-order Møller–Plesset perturbation theory (MP2)": {
        "method": "mp2",
        "calculation": ["energy", "gradients"],
        "level": "normal",
        "gradients": "analytic",
        "freeze core?": True,
    },
    "3rd-order Møller–Plesset perturbation theory (MP3)": {
        "method": "mp3",
        "calculation": ["energy", "gradients"],
        "level": "normal",
        "gradients": "analytic",
        "freeze core?": True,
    },
    "MP3 with frozen natural orbitals": {
        "method": "fno-mp3",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "average of MP2 and MP3": {
        "method": "mp2.5",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "analytic",
        "freeze core?": True,
    },
    "4th-order MP perturbation theory (MP4) less triples": {
        "method": "mp4(sdq)",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "MP4 (less triples) with frozen natural orbitals": {
        "method": "fno-mp4(sdq)",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "full MP4": {
        "method": "mp4",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "full MP4 with frozen natural orbitals": {
        "method": "fno-mp4",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "nth-order Møller–Plesset (MP) perturbation theory": {
        "method": "mpn",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "nth-order z-averaged perturbation theory (ZAPT)": {
        "method": "zaptn",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "orbital-optimized second-order MP perturbation theory": {
        "method": "omp2",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "analytic",
        "freeze core?": True,
    },
    "spin-component scaled OMP2": {
        "method": "scs-omp2",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "a special version of SCS-OMP2 for nucleobase interactions": {
        "method": "scs(n)-omp2",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "a special version of SCS-OMP2 (from ethene dimers)": {
        "method": "scs-omp2-vdw",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "spin-opposite scaled OMP2": {
        "method": "sos-omp2",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "A special version of SOS-OMP2 for pi systems": {
        "method": "sos-pi-omp2",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "orbital-optimized third-order MP perturbation theory": {
        "method": "omp3",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "analytic",
        "freeze core?": True,
    },
    "spin-component scaled OMP3": {
        "method": "scs-omp3",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "a special version of SCS-OMP3 for nucleobase interactions": {
        "method": "scs(n)-omp3",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "a special version of SCS-OMP3 (from ethene dimers)": {
        "method": "scs-omp3-vdw",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "spin-opposite scaled OMP3": {
        "method": "sos-omp3",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "A special version of SOS-OMP3 for pi systems": {
        "method": "sos-pi-omp3",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "orbital-optimized MP2.5": {
        "method": "omp2.5",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "analytic",
        "freeze core?": True,
    },
    "coupled electron pair approximation variant 0": {
        "method": "cepa(0)",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "CEPA(0) with frozen natural orbitals": {
        "method": "fno-cepa(0)",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "coupled electron pair approximation variant 1": {
        "method": "cepa(1)",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "CEPA(1) with frozen natural orbitals": {
        "method": "fno-cepa(1)",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "coupled electron pair approximation variant 3": {
        "method": "cepa(3)",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "CEPA(3) with frozen natural orbitals": {
        "method": "fno-cepa(3)",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "averaged coupled-pair functional": {
        "method": "acpf",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "ACPF with frozen natural orbitals": {
        "method": "fno-acpf",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "averaged quadratic coupled cluster": {
        "method": "aqcc",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "AQCC with frozen natural orbitals": {
        "method": "fno-aqcc",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "quadratic CI singles doubles (QCISD)": {
        "method": "qcisd",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "QCISD with frozen natural orbitals": {
        "method": "fno-qcisd",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "Linear CCD": {
        "method": "lccd",
        "calculation": ["energy", "gradients"],
        "level": "normal",
        "gradients": "analytic",
        "freeze core?": True,
    },
    "LCCD with frozen natural orbitals": {
        "method": "fno-lccd",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "orbital optimized LCCD": {
        "method": "olccd",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "analytic",
        "freeze core?": True,
    },
    "approximate coupled cluster singles and doubles (CC2)": {
        "method": "cc2",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "coupled cluster doubles (CCD)": {
        "method": "ccd",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "analytic",
        "freeze core?": True,
    },
    "coupled cluster singles and doubles (CCSD)": {
        "method": "ccsd",
        "calculation": ["energy", "gradients"],
        "level": "normal",
        "gradients": "analytic",
        "freeze core?": True,
    },
    "Brueckner coupled cluster doubles (BCCD)": {
        "method": "bccd",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "CCSD with frozen natural orbitals": {
        "method": "fno-ccsd",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "QCISD with perturbative triples": {
        "method": "qcisd(t)",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "QCISD(T) with frozen natural orbitals": {
        "method": "fno-qcisd(t)",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "CCSD with perturbative triples (CCSD(T))": {
        "method": "ccsd(t)",
        "calculation": ["energy", "gradients"],
        "level": "normal",
        "gradients": "analytic",
        "freeze core?": True,
    },
    "CCSD with asymmetric perturbative triples (CCSD(AT))": {
        "method": "ccsd(at)",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "BCCD with perturbative triples": {
        "method": "bccd(t)",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "CCSD(T) with frozen natural orbitals": {
        "method": "fno-ccsd(t)",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "approximate CC singles, doubles, and triples (CC3)": {
        "method": "cc3",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "expert full control over ccenergy module": {
        "method": "ccenergy",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "expert full control over dfocc module": {
        "method": "dfocc",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "configuration interaction (CI) singles and doubles (CISD)": {
        "method": "cisd",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "CISD with frozen natural orbitals": {
        "method": "fno-cisd",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "CI singles, doubles, and triples (CISDT)": {
        "method": "cisdt",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "CI singles, doubles, triples, and quadruples (CISDTQ)": {
        "method": "cisdtq",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "nth-order CI": {
        "method": "cin",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "full configuration interaction (FCI)": {
        "method": "fci",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "expert full control over detci module": {
        "method": "detci",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "complete active space self consistent field (CASSCF)": {
        "method": "casscf",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": False,
    },
    "restricted active space self consistent field (RASSCF)": {
        "method": "rasscf",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": False,
    },
    "multiconfigurational self consistent field (SCF)": {
        "method": "mcscf",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": False,
    },
    "Mukherjee multireference coupled cluster (Mk-MRCC)": {
        "method": "psimrcc",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": False,
    },
    "density matrix renormalization group SCF": {
        "method": "dmrg-scf",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": False,
    },
    "density matrix renormalization group CASPT2": {
        "method": "dmrg-caspt2",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "density matrix renormalization group CI": {
        "method": "dmrg-ci",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "0th-order symmetry adapted perturbation theory (SAPT)": {
        "method": "sapt0",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "0th-order SAPT with special exchange scaling": {
        "method": "ssapt0",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "0th-order functional and/or intramolecular SAPT": {
        "method": "fisapt0",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "2nd-order SAPT, traditional definition": {
        "method": "sapt2",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "SAPT including all 2nd-order terms": {
        "method": "sapt2+",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "SAPT including perturbative triples": {
        "method": "sapt2+(3)",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "SAPT including all 3rd-order terms": {
        "method": "sapt2+3",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "SAPT2+ with CC-based dispersion": {
        "method": "sapt2+(ccd)",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "SAPT2+(3) with CC-based dispersion": {
        "method": "sapt2+(3)(ccd)",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "SAPT2+3 with CC-based dispersion": {
        "method": "sapt2+3(ccd)",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "SAPT including all 2nd-order terms and MP2 correction": {
        "method": "sapt2+dmp2",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "SAPT including perturbative triples and MP2 correction": {
        "method": "sapt2+(3)dmp2",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "SAPT including all 3rd-order terms and MP2 correction": {
        "method": "sapt2+3dmp2",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "SAPT2+ with CC-based dispersion and MP2 correction": {
        "method": "sapt2+(ccd)dmp2",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "SAPT2+(3) with CC-based dispersion and MP2 correction": {
        "method": "sapt2+(3)(ccd)dmp2",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "SAPT2+3 with CC-based dispersion and MP2 correction": {
        "method": "sapt2+3(ccd)dmp2",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "0th-order SAPT plus charge transfer (CT) calculation": {
        "method": "sapt0-ct",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "SAPT2 plus CT": {
        "method": "sapt2-ct",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "SAPT2+ plus CT": {
        "method": "sapt2+-ct",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "SAPT2+(3) plus CT": {
        "method": "sapt2+(3)-ct",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "SAPT2+3 plus CT": {
        "method": "sapt2+3-ct",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "SAPT2+(CCD) plus CT": {
        "method": "sapt2+(ccd)-ct",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "SAPT2+(3)(CCD) plus CT": {
        "method": "sapt2+(3)(ccd)-ct",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "SAPT2+3(CCD) plus CT": {
        "method": "sapt2+3(ccd)-ct",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "2nd-order algebraic diagrammatic construction (ADC)": {
        "method": "adc",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "EOM-CC2": {
        "method": "eom-cc2",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
    "equation of motion (EOM) CCSD": {
        "method": "eom-ccsd",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "analytic",
        "freeze core?": True,
    },
    "EOM-CC3": {
        "method": "eom-cc3",
        "calculation": ["energy", "gradients"],
        "level": "expert",
        "gradients": "finite-difference",
        "freeze core?": True,
    },
}  # yapf: disable

dft_functionals = {
    "B1LYP Hyb-GGA Exchange-Correlation Functional": {
        "name": "b1lyp",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "B1PW91 Hyb-GGA Exchange-Correlation Functional": {
        "name": "b1pw91",
        "dispersion": ["none"],
        "level": "expert",
    },
    "B1WC Hyb-GGA Exchange-Correlation Functional": {
        "name": "b1wc",
        "dispersion": ["none"],
        "level": "expert",
    },
    "B2GPPLYP Double Hybrid Exchange-Correlation Functional": {
        "name": "b2gpplyp",
        "dispersion": ["none", "d3bj", "nl"],
        "level": "expert",
    },
    "B2PLYP Double Hybrid Exchange-Correlation Functional": {
        "name": "b2plyp",
        "dispersion": ["none", "d3bj", "d3mbj", "nl"],
        "level": "expert",
    },
    "B3LYP Hyb-GGA Exchange-Correlation Functional": {
        "name": "b3lyp",
        "dispersion": ["none", "d3bj", "d3mbj", "nl"],
        "level": "normal",
    },
    "B3LYP5 Hyb-GGA Exchange-Correlation Functional": {
        "name": "b3lyp5",
        "dispersion": ["none"],
        "level": "expert",
    },
    "B3LYPs Hyb-GGA Exchange-Correlation Functional": {
        "name": "b3lyps",
        "dispersion": ["none"],
        "level": "expert",
    },
    "B3P86 Hyb-GGA Exchange-Correlation Functional": {
        "name": "b3p86",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "B3PW91 Hyb-GGA Exchange-Correlation Functional": {
        "name": "b3pw91",
        "dispersion": ["none", "d3bj", "nl"],
        "level": "expert",
    },
    "B5050LYP Hyb-GGA Exchange-Correlation Functional": {
        "name": "b5050lyp",
        "dispersion": ["none"],
        "level": "expert",
    },
    "B86B95 Hyb-GGA Exchange-Correlation Functional": {
        "name": "b86b95",
        "dispersion": ["none"],
        "level": "expert",
    },
    "B86BPBE GGA Exchange-Correlation Functional": {
        "name": "b86bpbe",
        "dispersion": ["none"],
        "level": "expert",
    },
    "B88B95 Hyb-GGA Exchange-Correlation Functional": {
        "name": "b88b95",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "B97-0 Hyb-GGA Exchange-Correlation Functional": {
        "name": "b97-0",
        "dispersion": ["none"],
        "level": "expert",
    },
    "B97-1 Hyb-GGA Exchange-Correlation Functional": {
        "name": "b97-1",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "B97-1p Hyb-GGA Exchange-Correlation Functional": {
        "name": "b97-1p",
        "dispersion": ["none"],
        "level": "expert",
    },
    "B97-2 Hyb-GGA Exchange-Correlation Functional": {
        "name": "b97-2",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "B97-3 Hyb-GGA Exchange-Correlation Functional": {
        "name": "b97-3",
        "dispersion": ["none"],
        "level": "expert",
    },
    "b97-d": {
        "name": "b97-d",
        "dispersion": ["none", "d3bj", "d3mbj"],
        "level": "expert",
    },
    "B97-GGA1 GGA Exchange-Correlation Functional": {
        "name": "b97-gga1",
        "dispersion": ["none"],
        "level": "expert",
    },
    "B97-K Hyb-GGA Exchange-Correlation Functional": {
        "name": "b97-k",
        "dispersion": ["none"],
        "level": "expert",
    },
    "b97m-d3bj": {
        "name": "b97m-d3bj",
        "dispersion": ["none"],
        "level": "expert",
    },
    "B97M-V GGA Exchange-Correlation Functional": {
        "name": "b97m-v",
        "dispersion": ["none"],
        "level": "expert",
    },
    "BB1K Hyb-GGA Exchange-Correlation Functional": {
        "name": "bb1k",
        "dispersion": ["none"],
        "level": "expert",
    },
    "BHandH Hyb-GGA Exchange-Correlation Functional": {
        "name": "bhandh",
        "dispersion": ["none"],
        "level": "expert",
    },
    "BHandHLYP Hyb-GGA Exchange-Correlation Functional": {
        "name": "bhandhlyp",
        "dispersion": ["none"],
        "level": "expert",
    },
    "BLYP GGA Exchange-Correlation Functional": {
        "name": "blyp",
        "dispersion": ["none", "d3bj", "d3mbj", "nl"],
        "level": "expert",
    },
    "BOP GGA Exchange-Correlation Functional": {
        "name": "bop",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "BP86 GGA Exchange-Correlation Functional": {
        "name": "bp86",
        "dispersion": ["none", "d3bj", "d3mbj", "nl"],
        "level": "expert",
    },
    "CAM-B3LYP Hyb-GGA Exchange-Correlation Functional": {
        "name": "cam-b3lyp",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "CAP0 Hyb-GGA Exchange-Correlation Functional": {
        "name": "cap0",
        "dispersion": ["none"],
        "level": "expert",
    },
    "core-dsd-blyp": {
        "name": "core-dsd-blyp",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "Dispersionless Hybrid Meta-GGA XC Functional": {
        "name": "dldf",
        "dispersion": ["none"],
        "level": "expert",
    },
    "Dispersionless Hybrid Meta-GGA XC Functional + d09": {
        "name": "dldf+d09",
        "dispersion": ["none"],
        "level": "expert",
    },
    "Dispersionless Hybrid Meta-GGA XC Functional + d10": {
        "name": "dldf+d10",
        "dispersion": ["none"],
        "level": "expert",
    },
    "DSD-BLYP SCS Double Hybrid XC Functional": {
        "name": "dsd-blyp",
        "dispersion": ["none", "d3bj", "nl"],
        "level": "expert",
    },
    "DSD-PBEB95 SCS Double Hybrid Meta-GGA XC Functional": {
        "name": "dsd-pbeb95",
        "dispersion": ["none", "d3bj", "nl"],
        "level": "expert",
    },
    "DSD-PBEP86 SCS Double Hybrid XC Functional": {
        "name": "dsd-pbep86",
        "dispersion": ["none", "d3bj", "nl"],
        "level": "expert",
    },
    "DSD-PBEPBE SCS Double Hybrid XC Functional": {
        "name": "dsd-pbepbe",
        "dispersion": ["none", "d3bj", "nl"],
        "level": "expert",
    },
    "EDF1 GGA Exchange-Correlation Functional": {
        "name": "edf1",
        "dispersion": ["none"],
        "level": "expert",
    },
    "EDF2 Hyb-GGA Exchange-Correlation Functional": {
        "name": "edf2",
        "dispersion": ["none"],
        "level": "expert",
    },
    "FT97 GGA Exchange-Correlation Functional": {
        "name": "ft97",
        "dispersion": ["none"],
        "level": "expert",
    },
    "GAM GGA Minnesota Exchange-Correlation Functional": {
        "name": "gam",
        "dispersion": ["none"],
        "level": "expert",
    },
    "HCTH120 GGA Exchange-Correlation Functional": {
        "name": "hcth120",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "HCTH147 GGA Exchange-Correlation Functional": {
        "name": "hcth147",
        "dispersion": ["none"],
        "level": "expert",
    },
    "HCTH407 GGA Exchange-Correlation Functional": {
        "name": "hcth407",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "HCTH407P GGA Exchange-Correlation Functional": {
        "name": "hcth407p",
        "dispersion": ["none"],
        "level": "expert",
    },
    "HCTH93 GGA Exchange-Correlation Functional": {
        "name": "hcth93",
        "dispersion": ["none"],
        "level": "expert",
    },
    "HCTHP14 GGA Exchange-Correlation Functional": {
        "name": "hcthp14",
        "dispersion": ["none"],
        "level": "expert",
    },
    "HCTHP76 GGA Exchange-Correlation Functional": {
        "name": "hcthp76",
        "dispersion": ["none"],
        "level": "expert",
    },
    "hf": {
        "name": "hf",
        "dispersion": ["none", "d3bj", "nl"],
        "level": "expert",
    },
    "hf+d": {
        "name": "hf+d ",
        "dispersion": ["none"],
        "level": "expert",
    },
    "Hartree Fock based 3C composite method with minimal basis set, gCP and D3(BJ)": {  # noqa: E501
        "name": "hf3c",
        "dispersion": ["none"],
        "level": "expert",
    },
    "HJS-B88 Hyb-GGA Exchange-Correlation Functional": {
        "name": "hjs-b88",
        "dispersion": ["none"],
        "level": "expert",
    },
    "HJS-B97X Hyb-GGA Exchange-Correlation Functional": {
        "name": "hjs-b97x",
        "dispersion": ["none"],
        "level": "expert",
    },
    "HJS-PBE Hyb-GGA Exchange-Correlation Functional": {
        "name": "hjs-pbe",
        "dispersion": ["none"],
        "level": "expert",
    },
    "HJS-PBE-SOL Hyb-GGA Exchange-Correlation Functional": {
        "name": "hjs-pbe-sol",
        "dispersion": ["none"],
        "level": "expert",
    },
    "HPBEINT Hyb-GGA Exchange-Correlation Functional": {
        "name": "hpbeint",
        "dispersion": ["none"],
        "level": "expert",
    },
    "HSE03 Hyb-GGA Exchange-Correlation Functional": {
        "name": "hse03",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "HSE06 Hyb-GGA Exchange-Correlation Functional": {
        "name": "hse06",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "KSDT Exchange-Correlation Functional": {
        "name": "ksdt",
        "dispersion": ["none"],
        "level": "expert",
    },
    "KT2 GGA Exchange-Correlation Functional": {
        "name": "kt2",
        "dispersion": ["none"],
        "level": "expert",
    },
    "LC-VV10 GGA Exchange-Correlation Functional": {
        "name": "lc-vv10",
        "dispersion": ["none"],
        "level": "expert",
    },
    "LRC-WPBE GGA Exchange-Correlation Functional": {
        "name": "lrc-wpbe",
        "dispersion": ["none"],
        "level": "expert",
    },
    "LRC-wPBEh Hyb-GGA Exchange-Correlation Functional": {
        "name": "lrc-wpbeh",
        "dispersion": ["none"],
        "level": "expert",
    },
    "M05 Meta-GGA XC Functional": {
        "name": "m05",
        "dispersion": ["none"],
        "level": "expert",
    },
    "Heavily Parameterized Hybrid M05-2X Meta-GGA XC Functional": {
        "name": "m05-2x",
        "dispersion": ["none"],
        "level": "expert",
    },
    "M06 Meta-GGA XC Functional": {
        "name": "m06",
        "dispersion": ["none"],
        "level": "expert",
    },
    "Hybrid M06-2X Meta-GGA XC Functional": {
        "name": "m06-2x",
        "dispersion": ["none"],
        "level": "expert",
    },
    "Minnesota M06-HF Hybrid XC Functional": {
        "name": "m06-hf",
        "dispersion": ["none"],
        "level": "expert",
    },
    "M06-L Meta-GGA XC Functional": {
        "name": "m06-l",
        "dispersion": ["none"],
        "level": "expert",
    },
    "Minnesota M08-HX Hybrid XC Functional": {
        "name": "m08-hx",
        "dispersion": ["none"],
        "level": "expert",
    },
    "Minnesota M08-SO Hybrid XC Functional": {
        "name": "m08-so",
        "dispersion": ["none"],
        "level": "expert",
    },
    "M11 Meta-GGA XC Functional": {
        "name": "m11",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "M11-L Meta-GGA XC Functional": {
        "name": "m11-l",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "MB3LYP-RC04 Hyb-GGA Exchange-Correlation Functional": {
        "name": "mb3lyp-rc04",
        "dispersion": ["none"],
        "level": "expert",
    },
    "MGGA_MS0 Meta-GGA XC Functional": {
        "name": "mgga_ms0",
        "dispersion": ["none"],
        "level": "expert",
    },
    "MGGA_MS1 Meta-GGA XC Functional": {
        "name": "mgga_ms1",
        "dispersion": ["none"],
        "level": "expert",
    },
    "MGGA_MS2 Meta-GGA XC Functional": {
        "name": "mgga_ms2",
        "dispersion": ["none"],
        "level": "expert",
    },
    "MGGA_MS2h Hybrid Meta-GGA XC Functional": {
        "name": "mgga_ms2h",
        "dispersion": ["none"],
        "level": "expert",
    },
    "MGGA_MVS Meta-GGA XC Functional": {
        "name": "mgga_mvs",
        "dispersion": ["none"],
        "level": "expert",
    },
    "MGGA_MV2h Hybrid Meta-GGA XC Functional": {
        "name": "mgga_mvsh",
        "dispersion": ["none"],
        "level": "expert",
    },
    "MN12-L Meta-GGA XC Functional": {
        "name": "mn12-l",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "MN12-SX Meta-GGA Hybrid Screened Exchange-Correlation Functional": {
        "name": "mn12-sx",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "MN15 Hybrid Meta-GGA Exchange-Correlation Functional": {
        "name": "mn15",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "MN15-L Meta-GGA XC Functional": {
        "name": "mn15-l",
        "dispersion": ["none"],
        "level": "expert",
    },
    "MOHLYP GGA Exchange-Correlation Functional": {
        "name": "mohlyp",
        "dispersion": ["none"],
        "level": "expert",
    },
    "MOHLYP2 GGA Exchange-Correlation Functional": {
        "name": "mohlyp2",
        "dispersion": ["none"],
        "level": "expert",
    },
    "mPW1B95 Hyb-GGA Exchange-Correlation Functional": {
        "name": "mpw1b95",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "mPW1K Hyb-GGA Exchange-Correlation Functional": {
        "name": "mpw1k",
        "dispersion": ["none"],
        "level": "expert",
    },
    "mPW1LYP Hybrid GGA Exchange-Correlation Functional": {
        "name": "mpw1lyp",
        "dispersion": ["none"],
        "level": "expert",
    },
    "mPW1PBE Hybrid GGA Exchange-Correlation Functional": {
        "name": "mpw1pbe",
        "dispersion": ["none"],
        "level": "expert",
    },
    "mPW1PW Hyb-GGA Exchange-Correlation Functional": {
        "name": "mpw1pw",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "mPW3LYP Hyb-GGA Exchange-Correlation Functional": {
        "name": "mpw3lyp",
        "dispersion": ["none"],
        "level": "expert",
    },
    "mPW3PW Hyb-GGA Exchange-Correlation Functional": {
        "name": "mpw3pw",
        "dispersion": ["none"],
        "level": "expert",
    },
    "mPWB1K Hyb-GGA Exchange-Correlation Functional": {
        "name": "mpwb1k",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "mPWLYP1M Hyb-GGA Exchange-Correlation Functional": {
        "name": "mpwlyp1m",
        "dispersion": ["none"],
        "level": "expert",
    },
    "mPWLYP1W GGA Exchange-Correlation Functional": {
        "name": "mpwlyp1w",
        "dispersion": ["none"],
        "level": "expert",
    },
    "mPWPW GGA Exchange-Correlation Functional": {
        "name": "mpwpw",
        "dispersion": ["none"],
        "level": "expert",
    },
    "N12 nonseparable GGA Exchange-Correlation Functional": {
        "name": "n12",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "N12-SX Hybrid nonseparable GGA Exchange-Correlation Functional": {
        "name": "n12-sx",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "O3LYP Hyb-GGA Exchange-Correlation Functional": {
        "name": "o3lyp",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "oblyp-e": {
        "name": "oblyp-d",
        "dispersion": ["none"],
        "level": "expert",
    },
    "PBE GGA Exchange, One-parameter Progressive correlation Functional": {
        "name": "op-pbe",
        "dispersion": ["none"],
        "level": "expert",
    },
    "opbe-d": {
        "name": "opbe-d",
        "dispersion": ["none"],
        "level": "expert",
    },
    "opwlyp-d": {
        "name": "opwlyp-d",
        "dispersion": ["none"],
        "level": "expert",
    },
    "otpss-d": {
        "name": "otpss-d",
        "dispersion": ["none"],
        "level": "expert",
    },
    "PBE GGA Exchange-Correlation Functional": {
        "name": "pbe",
        "dispersion": ["none", "d3bj", "d3mbj", "nl"],
        "level": "normal",
    },
    "PBE0 Hyb-GGA Exchange-Correlation Functional": {
        "name": "pbe0",
        "dispersion": ["none", "d3bj", "d3mbj", "nl"],
        "level": "normal",
    },
    "PBE0-13 Hyb-GGA Exchange-Correlation Functional": {
        "name": "pbe0-13",
        "dispersion": ["none"],
        "level": "expert",
    },
    "PBE0-2 Double Hybrid Exchange-Correlation Functional": {
        "name": "pbe0-2",
        "dispersion": ["none"],
        "level": "expert",
    },
    "pbe0-dh": {
        "name": "pbe0-dh",
        "dispersion": ["none"],
        "level": "expert",
    },
    "PBE1W GGA Exchange-Correlation Functional": {
        "name": "pbe1w",
        "dispersion": ["none"],
        "level": "expert",
    },
    "PBE50 Hybrid GGA Exchange-Correlation Functional": {
        "name": "pbe50",
        "dispersion": ["none"],
        "level": "expert",
    },
    "PBE Hybrid based 3C composite method with a small basis set, gCP and D3(BJ)": {  # noqa: E501
        "name": "pbeh3c",
        "dispersion": ["none"],
        "level": "expert",
    },
    "PBELYP1W GGA Exchange-Correlation Functional": {
        "name": "pbelyp1w",
        "dispersion": ["none"],
        "level": "expert",
    },
    "PKZB Meta-GGA XC Functional": {
        "name": "pkzb",
        "dispersion": ["none"],
        "level": "expert",
    },
    "PTPSS SOS Double Hybrid XC Functional": {
        "name": "ptpss",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "PW6B95 Hybrid Meta-GGA XC Functional": {
        "name": "pw6b95",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "PW86B95 Hyb-GGA Exchange-Correlation Functional": {
        "name": "pw86b95",
        "dispersion": ["none"],
        "level": "expert",
    },
    "PW86PBE GGA Exchange-Correlation Functional": {
        "name": "pw86pbe",
        "dispersion": ["none"],
        "level": "expert",
    },
    "PW91 GGA Exchange-Correlation Functional": {
        "name": "pw91",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "PWB6K Hyb-GGA Exchange-Correlation Functional": {
        "name": "pwb6k",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "PWPB95 SOS Double Hybrid XC Functional": {
        "name": "pwpb95",
        "dispersion": ["none", "d3bj", "nl"],
        "level": "expert",
    },
    "revB3LYP Hyb-GGA Exchange-Correlation Functional": {
        "name": "revb3lyp",
        "dispersion": ["none"],
        "level": "expert",
    },
    "revPBE GGA Exchange-Correlation Functional": {
        "name": "revpbe",
        "dispersion": ["none", "d3bj", "nl"],
        "level": "expert",
    },
    "revPBE0 Hybrid GGA Exchange-Correlation Functional": {
        "name": "revpbe0",
        "dispersion": ["none", "d3bj", "nl"],
        "level": "expert",
    },
    "revised TPSS Meta-GGA XC Functional": {
        "name": "revtpss",
        "dispersion": ["none", "d3bj", "nl"],
        "level": "expert",
    },
    "revTPSSh Hyb-GGA Exchange-Correlation Functional": {
        "name": "revtpssh",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "RPBE GGA Exchange-Correlation Functional": {
        "name": "rpbe",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "SB98-1a Hyb-GGA Exchange-Correlation Functional": {
        "name": "sb98-1a",
        "dispersion": ["none"],
        "level": "expert",
    },
    "SB98-1b Hyb-GGA Exchange-Correlation Functional": {
        "name": "sb98-1b",
        "dispersion": ["none"],
        "level": "expert",
    },
    "SB98-1c Hyb-GGA Exchange-Correlation Functional": {
        "name": "sb98-1c",
        "dispersion": ["none"],
        "level": "expert",
    },
    "SB98-2a Hyb-GGA Exchange-Correlation Functional": {
        "name": "sb98-2a",
        "dispersion": ["none"],
        "level": "expert",
    },
    "SB98-2b Hyb-GGA Exchange-Correlation Functional": {
        "name": "sb98-2b",
        "dispersion": ["none"],
        "level": "expert",
    },
    "SB98-2c Hyb-GGA Exchange-Correlation Functional": {
        "name": "sb98-2c",
        "dispersion": ["none"],
        "level": "expert",
    },
    "SOGGA Exchange + PBE Correlation Functional": {
        "name": "sogga",
        "dispersion": ["none"],
        "level": "expert",
    },
    "SOGGA11 Exchange-Correlation Functional": {
        "name": "sogga11",
        "dispersion": ["none"],
        "level": "expert",
    },
    "SOGGA11-X Hybrid Exchange-Correlation Functional": {
        "name": "sogga11-x",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "SVWN3 (RPA) LSDA Functional": {
        "name": "svwn",
        "dispersion": ["none"],
        "level": "expert",
    },
    "TETER93 Exchange-Correlation Functional": {
        "name": "teter93",
        "dispersion": ["none"],
        "level": "expert",
    },
    "TH-FC GGA Exchange-Correlation Functional": {
        "name": "th-fc",
        "dispersion": ["none"],
        "level": "expert",
    },
    "TH-FCFO GGA Exchange-Correlation Functional": {
        "name": "th-fcfo",
        "dispersion": ["none"],
        "level": "expert",
    },
    "TH-FCO GGA Exchange-Correlation Functional": {
        "name": "th-fco",
        "dispersion": ["none"],
        "level": "expert",
    },
    "TH-FL GGA Exchange-Correlation Functional": {
        "name": "th-fl",
        "dispersion": ["none"],
        "level": "expert",
    },
    "TH1 GGA Exchange-Correlation Functional": {
        "name": "th1",
        "dispersion": ["none"],
        "level": "expert",
    },
    "TH2 GGA Exchange-Correlation Functional": {
        "name": "th2",
        "dispersion": ["none"],
        "level": "expert",
    },
    "TH3 GGA Exchange-Correlation Functional": {
        "name": "th3",
        "dispersion": ["none"],
        "level": "expert",
    },
    "TH4 GGA Exchange-Correlation Functional": {
        "name": "th4",
        "dispersion": ["none"],
        "level": "expert",
    },
    "TPSS Meta-GGA XC Functional": {
        "name": "tpss",
        "dispersion": ["none", "d3bj", "nl"],
        "level": "expert",
    },
    "TPSSh Hyb-GGA Exchange-Correlation Functional": {
        "name": "tpssh",
        "dispersion": ["none", "d3bj", "nl"],
        "level": "expert",
    },
    "TPSSLYP1W GGA Exchange-Correlation Functional": {
        "name": "tpsslyp1w",
        "dispersion": ["none"],
        "level": "expert",
    },
    "TUNED-CAM-B3LYP Hyb-GGA Exchange-Correlation Functional": {
        "name": "tuned-cam-b3lyp",
        "dispersion": ["none"],
        "level": "expert",
    },
    "VSXC Meta-GGA XC Functional": {
        "name": "vsxc",
        "dispersion": ["none"],
        "level": "expert",
    },
    "VV10 GGA Exchange-Correlation Functional": {
        "name": "vv10",
        "dispersion": ["none"],
        "level": "expert",
    },
    "wB97 GGA Exchange-Correlation Functional": {
        "name": "wb97",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "wB97M-V Hyb-GGA Exchange-Correlation Functional": {
        "name": "wb97m-v",
        "dispersion": ["none"],
        "level": "expert",
    },
    "wB97X Hyb-GGA Exchange-Correlation Functional": {
        "name": "wb97x",
        "dispersion": ["none", "d3bj", "d"],
        "level": "expert",
    },
    "wB97X-V Hyb-GGA Exchange-Correlation Functional": {
        "name": "wb97x-v",
        "dispersion": ["none"],
        "level": "expert",
    },
    "PBE SR-XC Functional (HJS Model)": {
        "name": "wpbe",
        "dispersion": ["none", "d3bj", "d3mbj"],
        "level": "expert",
    },
    "PBE0 SR-XC Functional (HJS Model)": {
        "name": "wpbe0",
        "dispersion": ["none"],
        "level": "expert",
    },
    "X1B95 Hyb-GGA Exchange-Correlation Functional": {
        "name": "x1b95",
        "dispersion": ["none"],
        "level": "expert",
    },
    "X3LYP Hyb-GGA Exchange-Correlation Functional": {
        "name": "x3lyp",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "XB1K Hyb-GGA Exchange-Correlation Functional": {
        "name": "xb1k",
        "dispersion": ["none"],
        "level": "expert",
    },
    "XLYP GGA Exchange-Correlation Functional": {
        "name": "xlyp",
        "dispersion": ["none", "d3bj"],
        "level": "expert",
    },
    "ZLP GGA Exchange-Correlation Functional": {
        "name": "zlp",
        "dispersion": ["none"],
        "level": "expert",
    },
}  # yapf: disable

optimization_methods = {
    "Rational Function Optimization": "RFO",
    "Newton-Raphson": "NR",
    "Steepest Descents": "SD",
    "Static Linesearch": "LINESEARCH_STATIC",
}

optimization_convergence = {
    "CFOUR": "CFOUR",
    "Gaussian loose": "GAU_LOOSE",
    "Gaussian": "GAU",
    "Gaussian tight": "GAU_TIGHT",
    "Gaussian very tight": "GAU_VERYTIGHT",
    "MOLPRO": "MOLPRO",
    "NWChem loose": "NWCHEM_LOOSE",
    "NWChem": "GAU",
    "NWChem tight": "GUA_TIGHT",
    "QChem": "QCHEM",
    "TURBOMOLE": "TURBOMOLE",
    "Interfragment tight": "INTERFRAG_TIGHT",
    "Custom": "",
}

"""Properties that Psi4 produces.
`metadata["results"]` describes the results that this step can produce. It is a
dictionary where the keys are the internal names of the results within this step, and
the values are a dictionary describing the result. For example::

    metadata["results"] = {
        "total_energy": {
            "calculation": [
                "energy",
                "optimization",
            ],
            "description": "The total energy",
            "dimensionality": "scalar",
            "methods": [
                "ccsd",
                "ccsd(t)",
                "dft",
                "hf",
            ],
            "property": "total energy#QuickMin#{model}",
            "type": "float",
            "units": "E_h",
        },
    }

Fields
______

calculation : [str]
    Optional metadata describing what subtype of the step produces this result.
    The subtypes are completely arbitrary, but often they are types of calculations
    which is why this is name `calculation`. To use this, the step or a substep
    define `self._calculation` as a value. That value is used to select only the
    results with that value in this field.

description : str
    A human-readable description of the result.

dimensionality : str
    The dimensions of the data. The value can be "scalar" or an array definition
    of the form "[dim1, dim2,...]". Symmetric tringular matrices are denoted
    "triangular[n,n]". The dimensions can be integers, other scalar
    results, or standard parameters such as `n_atoms`. For example, '[3]',
    [3, n_atoms], or "triangular[n_aos, n_aos]".

methods : str
    Optional metadata like the `calculation` data. `methods` provides a second
    level of filtering, often used for the Hamiltionian for *ab initio* calculations
    where some properties may or may not be calculated depending on the type of
    theory.

property : str
    An optional definition of the property for storing this result. Must be one of
    the standard properties defined either in SEAMM or in this steps property
    metadata in `data/properties.csv`.

type : str
    The type of the data: string, integer, or float.

units : str
    Optional units for the result. If present, the value should be in these units.
"""
metadata["results"] = {
    "energy": {
        "description": "The total energy",
        "dimensionality": "scalar",
        "property": "total energy#Psi4#{model}",
        "type": "float",
        "units": "E_h",
    },
    "gradients": {
        "description": "The gradients",
        "dimensionality": "[3, n_atoms]",
        "type": "float",
        "units": "E_h/a_0",
    },
    "(T) CORRECTION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["ccsd(t)"],
        "type": "float",
        "units": "",
    },
    "-D ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "dispersion correction energy",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
        "units": "E_h",
    },
    "-D GRADIENT": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "gradient of the dispersion correction energy",
        "dimensionality": [3, "n_atoms"],
        "methods": ["dft"],
        "type": "float",
        "units": "E_h/a_0",
    },
    "32-POLE": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "5th order electrical multipole",
        "dimensionality": ["triangular", 3, 3, 3, 3, 3],
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
    },
    "AAA (T) CORRECTION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["ccsd(t)"],
        "type": "float",
        "units": "E_h",
    },
    "AAB (T) CORRECTION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["ccsd(t)"],
        "type": "float",
        "units": "E_h",
    },
    "ABB (T) CORRECTION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["ccsd(t)"],
        "type": "float",
        "units": "E_h",
    },
    "B3LYP-D3(BJ) DISPERSION CORRECTION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "the dispersion correction energy",
        "dimensionality": "scalar",
        "methods": ["dft"],
        "type": "float",
        "units": "E_h",
    },
    "BBB (T) CORRECTION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["ccsd(t)"],
        "type": "float",
        "units": "E_h",
    },
    "CC D1 DIAGNOSTIC": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)"],
        "type": "float",
    },
    "CC D2 DIAGNOSTIC": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)"],
        "type": "float",
    },
    "CC NEW D1 DIAGNOSTIC": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)"],
        "type": "float",
    },
    "CC T1 DIAGNOSTIC": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)"],
        "type": "float",
    },
    "CCSD CORRELATION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "the correlation energy for a CCSD calculation",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)"],
        "type": "float",
        "units": "E_h",
    },
    "CCSD OPPOSITE-SPIN CORRELATION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)"],
        "type": "float",
        "units": "E_h",
    },
    "CCSD SAME-SPIN CORRELATION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)"],
        "type": "float",
        "units": "E_h",
    },
    "CCSD TOTAL ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "the total electronic energy from a CCSD calculation",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)"],
        "type": "float",
        "units": "E_h",
    },
    "CCSD(T) CORRELATION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["ccsd(t)"],
        "type": "float",
        "units": "E_h",
    },
    "CCSD(T) TOTAL ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["ccsd(t)"],
        "type": "float",
        "units": "E_h",
    },
    "CURRENT CORRELATION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "the correlation energy for the current method",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
        "units": "E_h",
    },
    "CURRENT DIPOLE": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "the electrical dipole moment for the current method",
        "dimensionality": [3],
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
        "units": "D",
    },
    "CURRENT DIPOLE X": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "the electrical dipole moment for the current method",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
        "units": "D",
    },
    "CURRENT DIPOLE Y": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "the electrical dipole moment for the current method",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
        "units": "D",
    },
    "CURRENT DIPOLE Z": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "the electrical dipole moment for the current method",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
        "units": "D",
    },
    "CURRENT ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "the electronic energy from the current method",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
        "units": "E_h",
    },
    "CURRENT GRADIENT": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "the gradient of the energy for the current method",
        "dimensionality": [3, "n_atoms"],
        "methods": ["dft", "hf"],
        "type": "float",
        "units": "E_h/a_0",
    },
    "CURRENT REFERENCE ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
        "units": "E_h",
    },
    "DFT FUNCTIONAL TOTAL ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "total energy of DFT functional",
        "dimensionality": "scalar",
        "methods": ["dft"],
        "type": "float",
        "units": "E_h",
    },
    "DFT TOTAL ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "Total DFT energy including dispersion",
        "dimensionality": "scalar",
        "methods": ["dft"],
        "type": "float",
        "units": "E_h",
    },
    "DFT VV10 ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "VV10 energy in DFT",
        "dimensionality": "scalar",
        "methods": ["dft"],
        "type": "float",
        "units": "E_h",
    },
    "DFT XC ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "DFT exchange-correlation energy",
        "dimensionality": "scalar",
        "methods": ["dft"],
        "type": "float",
        "units": "E_h",
    },
    "DIPOLE": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "electrical dipole moment",
        "dimensionality": [3],
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
        "units": "D",
    },
    "DIPOLE X": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "electrical dipole moment",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
        "units": "D",
    },
    "DIPOLE Y": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "electrical dipole moment",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
        "units": "D",
    },
    "DIPOLE Z": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "electrical dipole moment",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
        "units": "D",
    },
    "DISPERSION CORRECTION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "energy of the dispersion correction",
        "dimensionality": "scalar",
        "methods": ["dft"],
        "type": "float",
        "units": "E_h",
    },
    "ELECTROSTATIC POTENTIAL": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "the electrostatic potential at the nuclei",
        "dimensionality": ["n_atoms"],
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
        "units": "E_h",
    },
    "Eelec": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "the electronic energy",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "property": "electronic energy#Psi4#{model}",
        "type": "float",
        "units": "E_h",
    },
    "HEXADECAPOLE": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "electrical hexadecapole moment",
        "dimensionality": ["triangular", 3, 3, 3, 3],
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
    },
    "HF TOTAL ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "the Hartree-Fock electronic energy",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)", "hf", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
        "units": "E_h",
    },
    "LCCD CORRELATION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["lccd"],
        "type": "float",
        "units": "E_h",
    },
    "LCCD OPPOSITE-SPIN CORRELATION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["lccd"],
        "type": "float",
        "units": "E_h",
    },
    "LCCD SAME-SPIN CORRELATION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["lccd"],
        "type": "float",
        "units": "E_h",
    },
    "LCCD TOTAL ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["lccd"],
        "type": "float",
        "units": "E_h",
    },
    "LCCSD (+LMP2) TOTAL ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)"],
        "type": "float",
        "units": "E_h",
    },
    "LOWDIN_CHARGES": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "atomic charges using Lowdin's method",
        "dimensionality": ["n_atoms"],
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
        "units": "e",
    },
    "MAYER_INDICES": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "Mayer's bond indices",
        "dimensionality": ["triangular", "n_atoms", "n_atoms"],
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
    },
    "MP2 CORRELATION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
        "units": "E_h",
    },
    "MP2 OPPOSITE-SPIN CORRELATION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
        "units": "E_h",
    },
    "MP2 SAME-SPIN CORRELATION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
        "units": "E_h",
    },
    "MP2 SINGLES ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["mp2"],
        "type": "float",
        "units": "E_h",
    },
    "MP2 TOTAL ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
        "units": "E_h",
    },
    "MP2.5 CORRELATION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["mp3", "mp4"],
        "type": "float",
        "units": "E_h",
    },
    "MP2.5 TOTAL ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["mp3", "mp4"],
        "type": "float",
        "units": "E_h",
    },
    "MP3 CORRELATION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["mp3", "mp4"],
        "type": "float",
        "units": "E_h",
    },
    "MP3 TOTAL ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["mp3", "mp4"],
        "type": "float",
        "units": "E_h",
    },
    "MP4 CORRELATION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["mp4"],
        "type": "float",
        "units": "E_h",
    },
    "MP4 TOTAL ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["mp4"],
        "type": "float",
        "units": "E_h",
    },
    "MP4(SDQ) CORRELATION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["mp4"],
        "type": "float",
        "units": "E_h",
    },
    "MP4(SDQ) TOTAL ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["mp4"],
        "type": "float",
        "units": "E_h",
    },
    "MP4(SDTQ) CORRELATION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["mp4"],
        "type": "float",
        "units": "E_h",
    },
    "MP4(SDTQ) TOTAL ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["mp4"],
        "type": "float",
        "units": "E_h",
    },
    "MP4(T) CORRECTION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["mp4"],
        "type": "float",
        "units": "E_h",
    },
    "MULLIKEN_CHARGES": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "atomic charges using Mulliken's method",
        "dimensionality": ["n_atoms"],
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
        "units": "e",
    },
    "NUCLEAR REPULSION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "property": "nuclear repulsion energy",
        "type": "float",
        "units": "E_h",
    },
    "OCTUPOLE": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "electrical octupole moment",
        "dimensionality": ["triangular", 3, 3, 3],
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
    },
    "ONE-ELECTRON ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "the one-electron energy",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
        "units": "E_h",
    },
    "OPTIMIZATION ITERATIONS": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3"],
        "type": "float",
    },
    "PCM POLARIZATION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "the PCM polarization energy",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "property": "dielectric energy#Psi4#{model}",
        "type": "float",
        "units": "E_h",
    },
    "QUADRUPOLE": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "electrical quadrupole moment",
        "dimensionality": ["triangular", 3, 3],
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
    },
    "SCF DIPOLE": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "electrical dipole moment from SCF",
        "dimensionality": [3],
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
    },
    "SCF DIPOLE X": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "electrical dipole moment from SCF",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
    },
    "SCF DIPOLE Y": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "electrical dipole moment from SCF",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
    },
    "SCF DIPOLE Z": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "electrical dipole moment from SCF",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
    },
    "SCF ITERATION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
        "units": "E_h",
    },
    "SCF ITERATIONS": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "number of iterations in the SCF",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "type": "integer",
    },
    "SCF TOTAL ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
        "units": "E_h",
    },
    "SCF TOTAL GRADIENT": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": [3, "n_atoms"],
        "methods": ["dft", "hf"],
        "type": "float",
        "units": "E_h/a_0",
    },
    "SCS-MP2 CORRELATION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["lccd", "mp2", "mp3"],
        "type": "float",
        "units": "E_h",
    },
    "SCS-MP2 OPPOSITE-SPIN CORRELATION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["mp2"],
        "type": "float",
        "units": "E_h",
    },
    "SCS-MP2 SAME-SPIN CORRELATION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["mp2"],
        "type": "float",
        "units": "E_h",
    },
    "SCS-MP2 TOTAL ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["lccd", "mp2", "mp3"],
        "type": "float",
        "units": "E_h",
    },
    "SCS-MP2-VDW CORRELATION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["lccd", "mp3"],
        "type": "float",
        "units": "E_h",
    },
    "SCS-MP2-VDW TOTAL ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["lccd", "mp3"],
        "type": "float",
        "units": "E_h",
    },
    "SCS-MP3 CORRELATION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["mp3"],
        "type": "float",
        "units": "E_h",
    },
    "SCS-MP3 TOTAL ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["mp3"],
        "type": "float",
        "units": "E_h",
    },
    "SCS-MP3-VDW CORRELATION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["mp3"],
        "type": "float",
        "units": "E_h",
    },
    "SCS-MP3-VDW TOTAL ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["mp3"],
        "type": "float",
        "units": "E_h",
    },
    "SCSN-MP2 CORRELATION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["lccd", "mp3"],
        "type": "float",
        "units": "E_h",
    },
    "SCSN-MP2 TOTAL ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["lccd", "mp3"],
        "type": "float",
        "units": "E_h",
    },
    "SCSN-MP3 CORRELATION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["mp3"],
        "type": "float",
        "units": "E_h",
    },
    "SCSN-MP3 TOTAL ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["mp3"],
        "type": "float",
        "units": "E_h",
    },
    "SOS-MP2 CORRELATION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["lccd", "mp3"],
        "type": "float",
        "units": "E_h",
    },
    "SOS-MP2 TOTAL ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["lccd", "mp3"],
        "type": "float",
        "units": "E_h",
    },
    "SOS-MP3 CORRELATION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["mp3"],
        "type": "float",
        "units": "E_h",
    },
    "SOS-MP3 TOTAL ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["mp3"],
        "type": "float",
        "units": "E_h",
    },
    "SOS-PI-MP2 CORRELATION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["lccd", "mp3"],
        "type": "float",
        "units": "E_h",
    },
    "SOS-PI-MP2 TOTAL ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["lccd", "mp3"],
        "type": "float",
        "units": "E_h",
    },
    "SOS-PI-MP3 CORRELATION ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["mp3"],
        "type": "float",
        "units": "E_h",
    },
    "SOS-PI-MP3 TOTAL ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["mp3"],
        "type": "float",
        "units": "E_h",
    },
    "TWO-ELECTRON ENERGY": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "the two-electron energy",
        "dimensionality": "scalar",
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
        "units": "E_h",
    },
    "WIBERG_LOWDIN_INDICES": {
        "calculation": ["energy", "optimization", "thermochemistry", "vibrations"],
        "description": "the Wiber-Lowdin bond indices",
        "dimensionality": ["triangular", "n_atoms", "n_atoms"],
        "methods": ["ccsd", "ccsd(t)", "dft", "hf", "lccd", "mp2", "mp3", "mp4"],
        "type": "float",
    },
    "B": {
        "calculation": ["thermochemistry"],
        "description": "rotational constants",
        "dimensionality": [3],
        "type": "float",
        "units": "cm^-1",
    },
    "Cp_elec": {
        "calculation": ["thermochemistry"],
        "description": "electronic Cp",
        "dimensionality": "scalar",
        "type": "float",
        "units": "J/mol/K",
    },
    "Cp_rot": {
        "calculation": ["thermochemistry"],
        "description": "rotational Cp",
        "dimensionality": "scalar",
        "type": "float",
        "units": "J/mol/K",
    },
    "Cp_tot": {
        "calculation": ["thermochemistry"],
        "description": "total Cp",
        "dimensionality": "scalar",
        "type": "float",
        "units": "J/mol/K",
    },
    "Cp_trans": {
        "calculation": ["thermochemistry"],
        "description": "translational Cp",
        "dimensionality": "scalar",
        "type": "float",
        "units": "J/mol/K",
    },
    "Cp_vib": {
        "calculation": ["thermochemistry"],
        "description": "vibrational Cp",
        "dimensionality": "scalar",
        "type": "float",
        "units": "J/mol/K",
    },
    "Cv_elec": {
        "calculation": ["thermochemistry"],
        "description": "electronic Cv",
        "dimensionality": "scalar",
        "type": "float",
        "units": "J/mol/K",
    },
    "Cv_rot": {
        "calculation": ["thermochemistry"],
        "description": "rotational Cv",
        "dimensionality": "scalar",
        "type": "float",
        "units": "J/mol/K",
    },
    "Cv_tot": {
        "calculation": ["thermochemistry"],
        "description": "total Cv",
        "dimensionality": "scalar",
        "type": "float",
        "units": "J/mol/K",
    },
    "Cv_trans": {
        "calculation": ["thermochemistry"],
        "description": "translational Cv",
        "dimensionality": "scalar",
        "type": "float",
        "units": "J/mol/K",
    },
    "Cv_vib": {
        "calculation": ["thermochemistry"],
        "description": "vibrational Cv",
        "dimensionality": "scalar",
        "type": "float",
        "units": "J/mol/K",
    },
    "E0": {
        "calculation": ["thermochemistry"],
        "description": "electronic energy",
        "dimensionality": "scalar",
        "type": "float",
        "units": "E_h",
    },
    "E_corr": {
        "calculation": ["thermochemistry"],
        "description": "total correction to the energy",
        "dimensionality": "scalar",
        "type": "float",
        "units": "kJ/mol",
    },
    "E_elec": {
        "calculation": ["thermochemistry"],
        "description": "electronic correction to the energy",
        "dimensionality": "scalar",
        "type": "float",
        "units": "kJ/mol",
    },
    "E_rot": {
        "calculation": ["thermochemistry"],
        "description": "rotational correction to the energy",
        "dimensionality": "scalar",
        "type": "float",
        "units": "kJ/mol",
    },
    "E_tot": {
        "calculation": ["thermochemistry"],
        "description": "corrected energy",
        "dimensionality": "scalar",
        "type": "float",
        "units": "E_h",
    },
    "E_trans": {
        "calculation": ["thermochemistry"],
        "description": "translational correction to the energy",
        "dimensionality": "scalar",
        "type": "float",
        "units": "kJ/mol",
    },
    "E_vib": {
        "calculation": ["thermochemistry"],
        "description": "vibrational correction to the energy",
        "dimensionality": "scalar",
        "type": "float",
        "units": "kJ/mol",
    },
    "G_corr": {
        "calculation": ["thermochemistry"],
        "description": "total correction to the free energy",
        "dimensionality": "scalar",
        "type": "float",
        "units": "kJ/mol",
    },
    "G_elec": {
        "calculation": ["thermochemistry"],
        "description": "electronic correction to the free energy",
        "dimensionality": "scalar",
        "type": "float",
        "units": "kJ/mol",
    },
    "G_rot": {
        "calculation": ["thermochemistry"],
        "description": "rotational correction to the free energy",
        "dimensionality": "scalar",
        "type": "float",
        "units": "kJ/mol",
    },
    "G_tot": {
        "calculation": ["thermochemistry"],
        "description": "corrected free energy",
        "dimensionality": "scalar",
        "type": "float",
        "units": "E_h",
    },
    "G_trans": {
        "calculation": ["thermochemistry"],
        "description": "translational correction to the free energy",
        "dimensionality": "scalar",
        "type": "float",
        "units": "kJ/mol",
    },
    "G_vib": {
        "calculation": ["thermochemistry"],
        "description": "vibrational correction to the free energy",
        "dimensionality": "scalar",
        "type": "float",
        "units": "kJ/mol",
    },
    "H_corr": {
        "calculation": ["thermochemistry"],
        "description": "total correction to the enthalpy",
        "dimensionality": "scalar",
        "type": "float",
        "units": "kJ/mol",
    },
    "H_elec": {
        "calculation": ["thermochemistry"],
        "description": "electronic correction to the enthalpy",
        "dimensionality": "scalar",
        "type": "float",
        "units": "kJ/mol",
    },
    "H_rot": {
        "calculation": ["thermochemistry"],
        "description": "rotational correction to the enthalpy",
        "dimensionality": "scalar",
        "type": "float",
        "units": "kJ/mol",
    },
    "H_tot": {
        "calculation": ["thermochemistry"],
        "description": "corrected enthalpy",
        "dimensionality": "scalar",
        "type": "float",
        "units": "E_h",
    },
    "H_trans": {
        "calculation": ["thermochemistry"],
        "description": "translational correction to the enthalpy",
        "dimensionality": "scalar",
        "type": "float",
        "units": "kJ/mol",
    },
    "H_vib": {
        "calculation": ["thermochemistry"],
        "description": "vibrational correction to the enthalpy",
        "dimensionality": "scalar",
        "type": "float",
        "units": "kJ/mol",
    },
    "P": {
        "calculation": ["thermochemistry"],
        "description": "pressure",
        "dimensionality": "scalar",
        "type": "float",
        "units": "atm",
    },
    "S_corr": {
        "calculation": ["thermochemistry"],
        "description": "total correction to the entropy",
        "dimensionality": "scalar",
        "type": "float",
        "units": "J/mol/K",
    },
    "S_elec": {
        "calculation": ["thermochemistry"],
        "description": "electronic correction to the entropy",
        "dimensionality": "scalar",
        "type": "float",
        "units": "J/mol/K",
    },
    "S_rot": {
        "calculation": ["thermochemistry"],
        "description": "rotational correction to the entropy",
        "dimensionality": "scalar",
        "type": "float",
        "units": "J/mol/K",
    },
    "S_tot": {
        "calculation": ["thermochemistry"],
        "description": "corrected entropy",
        "dimensionality": "scalar",
        "type": "float",
        "units": "J/mol/K",
    },
    "S_trans": {
        "calculation": ["thermochemistry"],
        "description": "translational correction to the entropy",
        "dimensionality": "scalar",
        "type": "float",
        "units": "J/mol/K",
    },
    "S_vib": {
        "calculation": ["thermochemistry"],
        "description": "vibrational correction to the entropy",
        "dimensionality": "scalar",
        "type": "float",
        "units": "J/mol/K",
    },
    "T": {
        "calculation": ["thermochemistry"],
        "description": "temperature",
        "dimensionality": "scalar",
        "type": "float",
        "units": "K",
    },
    "ZPE_corr": {
        "calculation": ["thermochemistry"],
        "description": "total zero-point energy",
        "dimensionality": "scalar",
        "type": "float",
        "units": "kJ/mol",
    },
    "ZPE_elec": {
        "calculation": ["thermochemistry"],
        "description": "electronic zero-point energy",
        "dimensionality": "scalar",
        "type": "float",
        "units": "kJ/mol",
    },
    "ZPE_rot": {
        "calculation": ["thermochemistry"],
        "description": "rotational zero-point energy",
        "dimensionality": "scalar",
        "type": "float",
        "units": "kJ/mol",
    },
    "ZPE_tot": {
        "calculation": ["thermochemistry"],
        "description": "energy with zero-point",
        "dimensionality": "scalar",
        "type": "float",
        "units": "E_h",
    },
    "ZPE_trans": {
        "calculation": ["thermochemistry"],
        "description": "translational zero-point energy",
        "dimensionality": "scalar",
        "type": "float",
        "units": "kJ/mol",
    },
    "ZPE_vib": {
        "calculation": ["thermochemistry"],
        "description": "vibrational zero-point energy",
        "dimensionality": "scalar",
        "type": "float",
        "units": "kJ/mol",
    },
    "degeneracy": {
        "calculation": ["thermochemistry"],
        "description": "degeneracy for the modes",
        "dimensionality": ["n_atoms"],
        "type": "integer",
        "units": "",
    },
    "gamma": {
        "calculation": ["thermochemistry"],
        "description": "symmetry of the modes",
        "dimensionality": ["n_atoms"],
        "type": "string",
        "units": "",
    },
    "omega": {
        "calculation": ["thermochemistry"],
        "description": "vibrational frequencies",
        "dimensionality": ["n_atoms", 2],
        "type": "string",
        "units": "cm^-1",
    },
    "sigma": {
        "calculation": ["thermochemistry"],
        "description": "symmetry number",
        "dimensionality": "scalar",
        "type": "integer",
        "units": "",
    },
    # From cclib
    "moenergies": {
        "calculation": ["energy", "optimization"],
        "description": "The energy of the molecular orbitals (MOs)",
        "dimensionality": "[[nmo]*nspin]",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "eV",
        "format": ".3f",
    },
    "moments": {
        "calculation": ["energy", "optimization"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "",
        "format": ".6f",
    },
    "multipole_reference": {
        "calculation": ["energy", "optimization"],
        "description": "The reference point for the multipole expansions",
        "dimensionality": "[3]",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "Å",
        "format": ".6f",
    },
    "dipole_moment": {
        "calculation": ["energy", "optimization"],
        "description": "The dipole moment",
        "dimensionality": "[3]",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "debye",
        "format": ".6f",
    },
    "dipole_moment_magnitude": {
        "calculation": ["energy", "optimization"],
        "description": "The magnitude of the dipole moment",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "debye",
        "format": ".6f",
    },
    "quadrupole_moment": {
        "calculation": ["energy", "optimization"],
        "description": "The quadrupole moment",
        "dimensionality": "[6]",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "debye*Å",
        "format": ".6f",
    },
    "octapole_moment": {
        "calculation": ["energy", "optimization"],
        "description": "The octapole moment",
        "dimensionality": "[10]",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "debye*Å^2",
        "format": ".6f",
    },
    "hexadecapole_moment": {
        "calculation": ["energy", "optimization"],
        "description": "The hexadecapole moment",
        "dimensionality": "[15]",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "debye*Å^3",
        "format": ".6f",
    },
    "mosysm": {
        "calculation": ["energy", "optimization"],
        "description": "The symmetry of the MOs",
        "dimensionality": "[nmo]",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "string",
        "units": "",
        "format": "",
    },
    "nbasis": {
        "description": "Number of basis functions",
        "dimensionality": "scalar",
        "type": "integer",
        "units": "",
        "format": "",
    },
    "nmo": {
        "description": "Number of molecular orbitals",
        "dimensionality": "scalar",
        "type": "integer",
        "units": "",
        "format": "",
    },
    "rotconstants": {
        "calculation": ["energy", "optimization"],
        "description": "Rotational constants",
        "dimensionality": "[[3]*nsteps]",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "GHz",
        "format": ".3f",
    },
    "scfenergies": {
        "calculation": ["energy", "optimization"],
        "description": "The energies per step",
        "dimensionality": "[nsteps]",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "eV",
        "format": ".3f",
    },
}



# -*- coding: utf-8 -*-

"""The graphical part of a Psi4 step"""

import seamm
from seamm_util import ureg, Q_, units_class  # noqa: F401
import psi4_step  # noqa: F401
import pprint  # noqa: F401


class TkPsi4(seamm.TkNode):
    """
    The graphical part of a Psi4 step in a flowchart.

    Attributes
    ----------
    namespace : str
        The namespace of the current step.
    node : Node
        The corresponding node of the non-graphical flowchart
    dialog : Dialog
        The Pmw dialog object
    sub_tk_flowchart : TkFlowchart
        A graphical Flowchart representing a subflowchart
    self[widget] : dict
        A dictionary of tk widgets built using the information
        contained in Psi4_parameters.py

    See Also
    --------
    Psi4, TkPsi4,
    Psi4Parameters,
    """

    def __init__(
        self,
        tk_flowchart=None,
        node=None,
        namespace="org.molssi.seamm.psi4.tk",
        canvas=None,
        x=None,
        y=None,
        w=200,
        h=50,
    ):
        """
        Initialize a graphical node.

        Parameters
        ----------
        tk_flowchart: Tk_Flowchart
            The graphical flowchart that we are in.
        node: Node
            The non-graphical node for this step.
        namespace: str
            The stevedore namespace for finding sub-nodes.
        canvas: Canvas
           The Tk canvas to draw on.
        x: float
            The x position of the nodes center on the canvas.
        y: float
            The y position of the nodes cetner on the canvas.
        w: float
            The nodes graphical width, in pixels.
        h: float
            The nodes graphical height, in pixels.
        """
        self.namespace = namespace
        self.dialog = None

        super().__init__(
            tk_flowchart=tk_flowchart, node=node, canvas=canvas, x=x, y=y, w=w, h=h
        )
        self.create_dialog()

    def create_dialog(self):
        """
        Create the dialog. A set of widgets will be chosen by default
        based on what is specified in the
        Psi4_parameters module.

        See Also
        --------
        TkPsi4.reset_dialog
        """
        frame = super().create_dialog("Edit Psi4 Step")

        # The information about widgets is held in self['xxxx'], i.e. this
        # class is in part a dictionary of widgets. This makes accessing
        # the widgets easier and allows loops, etc.

        # make it large!
        screen_w = self.dialog.winfo_screenwidth()
        screen_h = self.dialog.winfo_screenheight()
        w = int(0.9 * screen_w)
        h = int(0.8 * screen_h)
        x = int(0.05 * screen_w / 2)
        y = int(0.1 * screen_h / 2)

        self.dialog.geometry("{}x{}+{}+{}".format(w, h, x, y))

        self.tk_subflowchart = seamm.TkFlowchart(
            master=frame, flowchart=self.node.subflowchart, namespace=self.namespace
        )
        self.tk_subflowchart.draw()

    def right_click(self, event):
        """
        Handles the right click event on the node.

        See Also
        --------
        TkPsi4.edit
        """

        super().right_click(event)
        self.popup_menu.add_command(label="Edit..", command=self.edit)

        self.popup_menu.tk_popup(event.x_root, event.y_root, 0)

    def edit(self):
        """Present a dialog for editing the Psi4 input

        See Also
        --------
        TkPsi4.right_click
        """

        if self.dialog is None:
            self.create_dialog()

        self.dialog.activate(geometry="centerscreenfirst")

    def handle_dialog(self, result):
        """Handle the closing of the edit dialog

        What to do depends on the button used to close the dialog. If
        the user closes it by clicking the 'x' of the dialog window,
        None is returned, which we take as equivalent to cancel.

        Parameters
        ----------
        result : None or str
            The value of this variable depends on what the button
            the user clicked.

        """

        if result is None or result == "Cancel":
            self.dialog.deactivate(result)
            return

        if result == "Help":
            # display help!!!
            return

        if result != "OK":
            self.dialog.deactivate(result)
            raise RuntimeError("Don't recognize dialog result '{}'".format(result))

        self.dialog.deactivate(result)

    def update_flowchart(self, tk_flowchart=None, flowchart=None):
        """Update the nongraphical flowchart.

        This is only used in nodes that contain sub-flowcharts
        What to do depends on the button used to close the dialog. If
        the user closes it by clicking the 'x' of the dialog window,
        None is returned, which we take as equivalent to cancel.

        Parameters
        ----------
        tk_flowchart : seamm.tk_Flowchart
            A graphical representation of the SEAMM Flowchart

        flowchart : seamm.Flowchart
            A non-graphical representation of the SEAMM Flowchart

        """

        super().update_flowchart(
            flowchart=self.node.subflowchart, tk_flowchart=self.tk_subflowchart
        )

    def from_flowchart(self, tk_flowchart=None, flowchart=None):
        """Recreate the graphics from the non-graphical flowchart.

        This is only used in nodes that contain sub-flowcharts.

        Parameters
        ----------
        tk_flowchart : seamm.tk_Flowchart
            A graphical representation of the SEAMM Flowchart
        flowchart : seamm.Flowchart
            A non-graphical representation of the SEAMM Flowchart

        """

        super().from_flowchart(
            flowchart=self.node.subflowchart, tk_flowchart=self.tk_subflowchart
        )

    def handle_help(self):
        """Shows the help to the user when click on help button."""
        print("Help not implemented yet for Psi4!")



# -*- coding: utf-8 -*-

"""The graphical part of a Psi4 Energy node"""

import logging
import tkinter as tk
import tkinter.ttk as ttk

import psi4_step
import seamm
import seamm_widgets as sw

logger = logging.getLogger(__name__)


class TkEnergy(seamm.TkNode):
    def __init__(
        self,
        tk_flowchart=None,
        node=None,
        canvas=None,
        x=120,
        y=20,
        w=200,
        h=50,
        my_logger=logger,
    ):
        """Initialize the graphical Tk Psi4 energy step

        Keyword arguments:
        """
        self.results_widgets = []

        # Set the logging level for this module if requested
        # if 'psi4_tk_energy_log_level' in self.options:
        #     my_logger.setLevel(self.options.psi4_tk_energy_log_level)
        #     my_logger.critical(
        #         'Set log level to {}'.format(
        #             self.options.psi4_tk_energy_log_level
        #         )
        #     )

        super().__init__(
            tk_flowchart=tk_flowchart,
            node=node,
            canvas=canvas,
            x=x,
            y=y,
            w=w,
            h=h,
            my_logger=my_logger,
        )

    def right_click(self, event):
        """Probably need to add our dialog..."""

        super().right_click(event)
        self.popup_menu.add_command(label="Edit..", command=self.edit)

        self.popup_menu.tk_popup(event.x_root, event.y_root, 0)

    def create_dialog(self, title="Edit Psi4 Energy Step"):
        """Create the dialog!"""
        self.logger.debug("Creating the dialog")
        frame = super().create_dialog(title=title, widget="notebook", results_tab=True)

        # Create a frame for the calculation control
        self["calculation"] = ttk.LabelFrame(
            frame,
            borderwidth=4,
            relief="sunken",
            text="Calculation",
            labelanchor="n",
            padding=10,
        )
        # Create a frame for the convergence control
        self["convergence"] = ttk.LabelFrame(
            frame,
            borderwidth=4,
            relief="sunken",
            text="SCF Convergence Control",
            labelanchor="n",
            padding=10,
        )

        # Create all the widgets
        P = self.node.parameters
        for key in (
            "level",
            "method",
            "advanced_method",
            "functional",
            "advanced_functional",
            "dispersion",
            "spin-restricted",
            "freeze-cores",
            "stability analysis",
        ):
            self[key] = P[key].widget(self["calculation"])
        for key in (
            "use damping",
            "damping percentage",
            "damping convergence",
            "use level shift",
            "level shift",
            "level shift convergence",
            "use soscf",
            "soscf starting convergence",
            "soscf convergence",
            "soscf max iterations",
            "soscf print iterations",
            "density convergence",
            "energy convergence",
            "convergence error",
            "maximum iterations",
        ):
            self[key] = P[key].widget(self["convergence"])

        # bindings...
        self["level"].combobox.bind("<<ComboboxSelected>>", self.reset_calculation)
        self["level"].config(state="readonly")

        self["method"].combobox.bind("<<ComboboxSelected>>", self.reset_calculation)
        self["method"].combobox.bind("<Return>", self.reset_calculation)
        self["method"].combobox.bind("<FocusOut>", self.reset_calculation)

        self["advanced_method"].combobox.bind(
            "<<ComboboxSelected>>", self.reset_calculation
        )
        self["advanced_method"].combobox.bind("<Return>", self.reset_calculation)
        self["advanced_method"].combobox.bind("<FocusOut>", self.reset_calculation)

        for key in ("use damping", "use level shift", "use soscf"):
            self[key].bind("<<ComboboxSelected>>", self.reset_convergence)
            self[key].bind("<Return>", self.reset_convergence)
            self[key].bind("<FocusOut>", self.reset_convergence)

        # A tab for output -- orbitals, etc.
        notebook = self["notebook"]
        self["output frame"] = oframe = ttk.Frame(notebook)
        notebook.insert(self["results frame"], oframe, text="Output", sticky="new")

        # Frame to isolate widgets
        p_frame = self["plot frame"] = ttk.LabelFrame(
            self["output frame"],
            borderwidth=4,
            relief="sunken",
            text="Plots",
            labelanchor="n",
            padding=10,
        )

        for key in psi4_step.EnergyParameters.output:
            self[key] = P[key].widget(p_frame)

        # Set the callbacks for changes
        for widget in ("orbitals",):
            w = self[widget]
            w.combobox.bind("<<ComboboxSelected>>", self.reset_plotting)
            w.combobox.bind("<Return>", self.reset_plotting)
            w.combobox.bind("<FocusOut>", self.reset_plotting)
        p_frame.grid(row=0, column=0, sticky="new")
        oframe.columnconfigure(0, weight=1)

        self.reset_plotting()

        # Top level needs to call reset_dialog
        if self.node.calculation == "energy":
            self.reset_dialog()

        self.logger.debug("Finished creating the dialog")

        return frame

    def reset_dialog(self, row=0, widget=None):
        """Layout the widgets as needed for the current state"""

        frame = self["frame"]
        if row == 0:
            for slave in frame.grid_slaves():
                slave.grid_forget()

        self["calculation"].grid(row=row, column=0)
        self.reset_calculation()
        row += 1
        self["convergence"].grid(row=row, column=0)
        self.reset_convergence()
        row += 1
        return row

    def reset_calculation(self, widget=None):
        level = self["level"].get()

        if level == "recommended":
            long_method = self["method"].get()
            if self.is_expr(long_method):
                self.node.method = None
                meta = None
            else:
                self.node.method = psi4_step.methods[long_method]["method"]
                meta = psi4_step.methods[long_method]
            functional = self["functional"].get()
        else:
            long_method = self["advanced_method"].get()
            if self.is_expr(long_method):
                self.node.method = None
                meta = None
            else:
                self.node.method = psi4_step.methods[long_method]["method"]
                meta = psi4_step.methods[long_method]
            functional = self["advanced_functional"].get()

        # Set up the results table because it depends on the method
        self.results_widgets = []
        self.setup_results()

        frame = self["calculation"]
        for slave in frame.grid_slaves():
            slave.grid_forget()

        widgets = []
        widgets2 = []
        row = 0
        self["level"].grid(row=row, column=0, columnspan=2, sticky=tk.EW)
        row += 1
        if level == "recommended":
            self["method"].grid(row=row, column=0, columnspan=2, sticky=tk.EW)
            widgets.append(self["method"])
            row += 1
            if self.node.method is None or self.node.method == "dft":
                self["functional"].grid(row=row, column=1, sticky=tk.EW)
                widgets2.append(self["functional"])
                row += 1
            if meta is None or "freeze core?" in meta and meta["freeze core?"]:
                self["freeze-cores"].grid(row=row, column=1, sticky=tk.EW)
                widgets2.append(self["freeze-cores"])
                row += 1
        else:
            self["advanced_method"].grid(row=row, column=0, columnspan=2, sticky=tk.EW)
            widgets.append(self["advanced_method"])
            row += 1
            if self.node.method is None or self.node.method == "dft":
                self["advanced_functional"].grid(row=row, column=1, sticky=tk.EW)
                widgets2.append(self["advanced_functional"])
                row += 1
            if meta is None or "freeze core?" in meta and meta["freeze core?"]:
                self["freeze-cores"].grid(row=row, column=1, sticky=tk.EW)
                widgets2.append(self["freeze-cores"])
                row += 1
        if self.node.method is None or self.node.method == "dft":
            if functional in psi4_step.dft_functionals:
                dispersions = psi4_step.dft_functionals[functional]["dispersion"]
                if len(dispersions) > 1:
                    w = self["dispersion"]
                    w.config(values=dispersions)
                    if w.get() not in dispersions:
                        w.value(dispersions[1])
                    w.grid(row=row, column=1, sticky=tk.W)
                    widgets2.append(self["dispersion"])
                    row += 1
                sw.align_labels(widgets2)
                frame.columnconfigure(0, minsize=30)
        self["spin-restricted"].grid(row=row, column=0, columnspan=2, sticky=tk.EW)
        widgets.append(self["spin-restricted"])
        row += 1
        self["stability analysis"].grid(row=row, column=0, columnspan=2, sticky=tk.EW)
        widgets.append(self["stability analysis"])
        row += 1
        sw.align_labels(widgets, sticky=tk.E)

        return row

    def reset_convergence(self, widget=None):
        """Layout the convergence widgets as needed for the current state"""

        frame = self["convergence"]
        for slave in frame.grid_slaves():
            slave.grid_forget()

        widgets = []
        widgets2 = []
        row = 0

        for key in (
            "maximum iterations",
            "density convergence",
            "energy convergence",
            "convergence error",
            "use damping",
        ):
            self[key].grid(row=row, column=0, columnspan=2, sticky=tk.EW)
            widgets.append(self[key])
            row += 1

        if self["use damping"].get() != "no":
            for key in ("damping percentage", "damping convergence"):
                self[key].grid(row=row, column=1, sticky=tk.EW)
                widgets2.append(self[key])
                row += 1

        for key in ("use level shift",):
            self[key].grid(row=row, column=0, columnspan=2, sticky=tk.EW)
            widgets.append(self[key])
            row += 1
        if self["use level shift"].get() != "no":
            for key in ("level shift", "level shift convergence"):
                self[key].grid(row=row, column=1, sticky=tk.EW)
                widgets2.append(self[key])
                row += 1

        for key in ("use soscf",):
            self[key].grid(row=row, column=0, columnspan=2, sticky=tk.EW)
            widgets.append(self[key])
            row += 1
        if self["use soscf"].get() != "no":
            for key in (
                "soscf starting convergence",
                "soscf convergence",
                "soscf max iterations",
                "soscf print iterations",
            ):
                self[key].grid(row=row, column=1, sticky=tk.EW)
                widgets2.append(self[key])
                row += 1

        frame.columnconfigure(0, minsize=150)
        sw.align_labels(widgets, sticky=tk.E)
        sw.align_labels(widgets2, sticky=tk.E)

    def reset_plotting(self, widget=None):
        frame = self["plot frame"]
        for slave in frame.grid_slaves():
            slave.grid_forget()

        plot_orbitals = self["orbitals"].get() == "yes"

        widgets = []

        row = 0
        for key in (
            "density",
            "orbitals",
        ):
            self[key].grid(row=row, column=0, columnspan=4, sticky=tk.EW)
            widgets.append(self[key])
            row += 1

        if plot_orbitals:
            key = "selected orbitals"
            self[key].grid(row=row, column=1, columnspan=4, sticky=tk.EW)
            row += 1

        sw.align_labels(widgets, sticky=tk.E)
        frame.columnconfigure(0, minsize=50)
        frame.columnconfigure(4, weight=1)

        return row



# -*- coding: utf-8 -*-
"""Global control parameters for Psi4
"""

import logging

import psi4_step
from seamm.standard_parameters import structure_handling_parameters

# import seamm

logger = logging.getLogger(__name__)


class AcceleratedOptimizationParameters(psi4_step.EnergyParameters):
    """The control parameters for the MOPAC accelerated optimization."""

    parameters = {
        "optimization method": {
            "default": "RFO",
            "kind": "enumeration",
            "default_units": "",
            "enumeration": (
                "RFO",
                "P_RFO",
                "NR",
                "SD",
                "LINESEARCH",
            ),
            "format_string": "s",
            "description": "Method:",
            "help_text": "The optimization method to use.",
        },
        "max geometry steps": {
            "default": "default",
            "kind": "integer",
            "default_units": "",
            "enumeration": ("default",),
            "format_string": "",
            "description": "Maximum steps:",
            "help_text": (
                "The maximum number of steps to take in the optimization. "
                "'default' is based on the system size, giving a reasonable "
                "limit in most cases."
            ),
        },
        "geometry convergence": {
            "default": "QCHEM",
            "kind": "float",
            "default_units": "",
            "enumeration": (
                "QCHEM",
                "MOLPRO",
                "GAU",
                "GAU_LOOSE",
                "GAU_TIGHT",
                "GAU_VERYTIGHT",
                "TURBOMOLE",
                "CFOUR",
                "NWCHEM_LOOSE",
            ),
            "format_string": "",
            "description": "Convergence criteria:",
            "help_text": "The criteria to use for convergence.",
        },
        "recalc hessian": {
            "default": "every step",
            "kind": "integer",
            "default_units": "",
            "enumeration": ("every step", "at beginning", "never"),
            "format_string": "",
            "description": "Recalculate Hessian:",
            "help_text": (
                "How often to recalculate the Hessian (in steps). Smaller "
                "values help convergence but are expensive."
            ),
        },
        "hessian update": {
            "default": "bfgs",
            "kind": "enumeration",
            "default_units": "",
            "enumeration": (
                "bfgs",
                "ms",
                "powell",
                "none",
            ),
            "format_string": "s",
            "description": "Hessian update:",
            "help_text": "The algorithm for updating the Hessian.",
        },
        "coordinates": {
            "default": "Internal",
            "kind": "enumeration",
            "default_units": "",
            "enumeration": (
                "Internal",
                "Delocalized",
                "Natural",
                "Cartesian",
                "Both",
            ),
            "format_string": "s",
            "description": "Type of coordinates:",
            "help_text": "The type of coordinates to use in the minimization.",
        },
    }

    def __init__(self, defaults={}, data=None):
        """Initialize the instance, by default from the default
        parameters given in the class"""

        super().__init__(
            defaults={
                **AcceleratedOptimizationParameters.parameters,
                **structure_handling_parameters,
                **defaults,
            },
            data=data,
        )



# -*- coding: utf-8 -*-

"""Installer for the Psi4 plug-in.

This handles any further installation needed after installing the Python
package `psi4-step`.
"""

import logging
from pathlib import Path
import pkg_resources
import subprocess

import seamm_installer

logger = logging.getLogger(__name__)


class Installer(seamm_installer.InstallerBase):
    """Handle further installation needed after installing psi4-step.

    The Python package `psi4-step` should already be installed, using `pip`,
    `conda`, or similar. This plug-in-specific installer then checks for the
    Psi4 executable, installing it if needed, and registers its
    location in seamm.ini.

    There are a number of ways to determine which are the correct Psi4
    executables to use. The aim of this installer is to help the user locate
    the executables. There are a number of possibilities

    1. The correct executables are already available.

        #. If they are already registered in `seamm.ini` there is nothing else
           to do.

        #. They may be in the current path, in which case they need to be added
           to `seamm.ini`.

        #. If a module system is in use, a module may need to be loaded to give
           access to Psi4.

        #. They cannot be found automatically, so the user needs to locate the
           executables for the installer.

    #. Psi4 is not installed on the machine. In this case they can be
       installed in a Conda environment. There is one choice

        #. They can be installed in a separate environment, `seamm-psi4` by
           default.
    """

    def __init__(self, logger=logger):
        # Call the base class initialization, which sets up the commandline
        # parser, amongst other things.
        super().__init__(logger=logger)

        logger.debug("Initializing the Psi4 installer object.")

        self.section = "psi4-step"
        self.executables = ["psi4"]
        self.resource_path = Path(pkg_resources.resource_filename(__name__, "data/"))

        # The environment.yaml file for Conda installations.
        logger.debug(f"data directory: {self.resource_path}")
        self.environment_file = self.resource_path / "seamm-psi4.yml"

    def check(self):
        """Check the status of the Psi4 installation."""
        print("Checking the Psi4 installation.")

        # What Conda environment is the default?
        path = self.configuration.path.parent / "psi4.ini"
        if not path.exists():
            text = (self.resource_path / "psi4.ini").read_text()
            path.write_text(text)
            print(f"    The psi4.ini file did not exist. Created {path}")

        self.exe_config.path = path

        # Get the current values
        data = self.exe_config.get_values("local")

        if "conda-environment" in data and data["conda-environment"] != "":
            self.environment = data["conda-environment"]
        else:
            self.environment = "seamm-psi4"

        super().check()

    def install(self):
        """Install Psi4 in a conda environment."""
        print("Installing Psi4.")

        # What Conda environment is the default?
        path = self.configuration.path.parent / "psi4.ini"
        if not path.exists():
            text = (self.resource_path / "psi4.ini").read_text()
            path.write_text(text)
            print(f"    The psi4.ini file did not exist. Created {path}")

        self.exe_config.path = path

        # Get the current values
        data = self.exe_config.get_values("local")

        if "conda-environment" in data and data["conda-environment"] != "":
            self.environment = data["conda-environment"]
        else:
            self.environment = "seamm-psi4"

        super().install()

    def show(self):
        """Show the status of the Psi4 installation."""
        print("Showing the Psi4 installation.")

        # What Conda environment is the default?
        path = self.configuration.path.parent / "psi4.ini"
        if not path.exists():
            text = (self.resource_path / "psi4.ini").read_text()
            path.write_text(text)
            print(f"    The psi4.ini file does not exist at {path}")
            print("    The 'check' command will create it if Psi4 is installed.")
            print("    Otherwise 'install' will install Psi4.")
            return

        self.exe_config.path = path

        if not self.exe_config.section_exists("local"):
            print(
                "    Psi4 is not configured: there is no 'local' section in "
                f"     {path}."
            )
            return

        # Get the current values
        data = self.exe_config.get_values("local")

        if "conda-environment" in data and data["conda-environment"] != "":
            self.environment = data["conda-environment"]
        else:
            self.environment = "seamm-psi4"

        super().show()

    def uninstall(self):
        """Uninstall the Psi4 installation."""
        print("Uninstall the Psi4 installation.")

        # What Conda environment is the default?
        path = self.configuration.path.parent / "psi4.ini"
        if not path.exists():
            text = (self.resource_path / "psi4.ini").read_text()
            path.write_text(text)
            print(
                f""""    The psi4.ini file does not exist at {path}
    Perhaps Psi4 is not installed, but if it is the 'check' command may locate it
    and create the ini file, after which 'uninstall' will remove it."""
            )
            return

        self.exe_config.path = path

        if not self.exe_config.section_exists("local"):
            print(
                f""""    The psi4.ini file at {path} does not have local section.
    Perhaps Psi4 not installed, but if it is the 'check' command may locate it
    and update the ini file, after which 'uninstall' will remove it."""
            )
            return

        # Get the current values
        data = self.exe_config.get_values("local")

        if "conda-environment" in data and data["conda-environment"] != "":
            self.environment = data["conda-environment"]
        else:
            self.environment = "seamm-psi4"

        super().uninstall()

    def update(self):
        """Updates the Psi4 installation."""
        print("Updating the Psi4 installation.")

        # What Conda environment is the default?
        path = self.configuration.path.parent / "psi4.ini"
        if not path.exists():
            text = (self.resource_path / "psi4.ini").read_text()
            path.write_text(text)
            print(f"    The psi4.ini file did not exist. Created {path}")

        self.exe_config.path = path

        # Get the current values
        data = self.exe_config.get_values("local")

        if "conda-environment" in data and data["conda-environment"] != "":
            self.environment = data["conda-environment"]
        else:
            self.environment = "seamm-psi4"

        super().update()

    def exe_version(self, config):
        """Get the version of the Psi4 executable.

        Parameters
        ----------
        config : dict
            Dictionary of configuration parameters from psi4.ini

        Returns
        -------
        str, str
            "Psi4" and the version reported by the executable, or 'unknown'.
        """
        environment = config["conda-environment"]
        conda = config["conda"]
        if environment[0] == "~":
            environment = str(Path(environment).expanduser())
            command = f"'{conda}' run --live-stream -p '{environment}' psi4 --version"
        elif Path(environment).is_absolute():
            command = f"'{conda}' run --live-stream -p '{environment}' psi4 --version"
        else:
            command = f"'{conda}' run --live-stream -n '{environment}' psi4 --version"

        logger.debug(f"    Running {command}")
        try:
            result = subprocess.run(
                command,
                stdin=subprocess.DEVNULL,
                capture_output=True,
                text=True,
                shell=True,
            )
        except Exception as e:
            logger.debug(f"    Failed to run {command}: {e}")
            version = "unknown"
        else:
            version = "unknown"
            lines = result.stdout.splitlines()
            for line in lines:
                line = line.strip()
                tmp = line.split()
                if len(tmp) == 1:
                    version = tmp[0]
                    break

        # Psi4 tends to leave timing.dat floating around. :-(
        timing = Path("timer.dat")
        if timing.exists():
            try:
                timing.unlink()
            except Exception:
                pass

        return "Psi4", version
